@@ -1,18 +1,5 @@
-// ======================================================================== //
-// Copyright 2009-2020 Intel Corporation                                    //
-//                                                                          //
-// Licensed under the Apache License, Version 2.0 (the "License");          //
-// you may not use this file except in compliance with the License.         //
-// You may obtain a copy of the License at                                  //
-//                                                                          //
-//     http://www.apache.org/licenses/LICENSE-2.0                           //
-//                                                                          //
-// Unless required by applicable law or agreed to in writing, software      //
-// distributed under the License is distributed on an "AS IS" BASIS,        //
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. //
-// See the License for the specific language governing permissions and      //
-// limitations under the License.                                           //
-// ======================================================================== //
+// Copyright 2009-2020 Intel Corporation
+// SPDX-License-Identifier: Apache-2.0
 
 #pragma once
 
@@ -32,7 +19,7 @@ namespace embree
     __forceinline PrimRefMB () {}
 
     __forceinline PrimRefMB (const LBBox3fa& lbounds_i, unsigned int activeTimeSegments, BBox1f time_range, unsigned int totalTimeSegments, unsigned int geomID, unsigned int primID)
-      : lbounds(lbounds_i), time_range(time_range)
+      : lbounds((LBBox3fx)lbounds_i), time_range(time_range)
     {
       assert(activeTimeSegments > 0);
       lbounds.bounds0.lower.a = geomID;
@@ -42,7 +29,7 @@ namespace embree
     }
 
     __forceinline PrimRefMB (EmptyTy empty, const LBBox3fa& lbounds_i, unsigned int activeTimeSegments, BBox1f time_range, unsigned int totalTimeSegments, size_t id)
-      : lbounds(lbounds_i), time_range(time_range)
+      : lbounds((LBBox3fx)lbounds_i), time_range(time_range)
     {
       assert(activeTimeSegments > 0);
 #if defined(__X86_64__)
@@ -57,7 +44,7 @@ namespace embree
     }
     
     __forceinline PrimRefMB (const LBBox3fa& lbounds_i, unsigned int activeTimeSegments, BBox1f time_range, unsigned int totalTimeSegments, size_t id)
-      : lbounds(lbounds_i), time_range(time_range)
+      : lbounds((LBBox3fx)lbounds_i), time_range(time_range)
     {
       assert(activeTimeSegments > 0);
 #if defined(__X86_64__)
@@ -146,12 +133,12 @@ namespace embree
     }
 
     /*! Outputs primitive reference to a stream. */
-    friend __forceinline std::ostream& operator<<(std::ostream& cout, const PrimRefMB& ref) {
+    friend __forceinline embree_ostream operator<<(embree_ostream cout, const PrimRefMB& ref) {
       return cout << "{ bounds = " << ref.bounds() << ", geomID = " << ref.geomID() << ", primID = " << ref.primID() << ", active_segments = " << ref.size() << ",  total_segments = " << ref.totalTimeSegments() << " }";
     }
 
   public:
-    LBBox3fa lbounds;
+    LBBox3fx lbounds;
     BBox1f time_range; // entire geometry time range
   };
 
@@ -260,7 +247,7 @@ namespace embree
     }
 
     /*! Outputs primitive reference to a stream. */
-    friend __forceinline std::ostream& operator<<(std::ostream& cout, const PrimRefMB& ref) {
+    friend __forceinline embree_ostream operator<<(embree_ostream cout, const PrimRefMB& ref) {
       return cout << "{ bounds = " << ref.bounds() << ", geomID = " << ref.geomID() << ", primID = " << ref.primID() << ", active_segments = " << ref.size() << ",  total_segments = " << ref.totalTimeSegments() << " }";
     }
 
