@@ -1,18 +1,5 @@
-// ======================================================================== //
-// Copyright 2009-2020 Intel Corporation                                    //
-//                                                                          //
-// Licensed under the Apache License, Version 2.0 (the "License");          //
-// you may not use this file except in compliance with the License.         //
-// You may obtain a copy of the License at                                  //
-//                                                                          //
-//     http://www.apache.org/licenses/LICENSE-2.0                           //
-//                                                                          //
-// Unless required by applicable law or agreed to in writing, software      //
-// distributed under the License is distributed on an "AS IS" BASIS,        //
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. //
-// See the License for the specific language governing permissions and      //
-// limitations under the License.                                           //
-// ======================================================================== //
+// Copyright 2009-2020 Intel Corporation
+// SPDX-License-Identifier: Apache-2.0
 
 #pragma once
 
@@ -58,7 +45,11 @@ namespace embree
                          const vuint<M>& v3,
                          const vuint<M>& geomIDs,
                          const vuint<M>& primIDs)
-      : v0(v0),v1(v1), v2(v2), v3(v3), geomIDs(geomIDs), primIDs(primIDs) {}
+#if defined(EMBREE_COMPACT_POLYS)
+      : geomIDs(geomIDs), primIDs(primIDs) {}
+#else
+     : v0_(v0),v1_(v1), v2_(v2), v3_(v3), geomIDs(geomIDs), primIDs(primIDs) {}
+#endif
 
     /* Returns a mask that tells which quads are valid */
     __forceinline vbool<M> valid() const { return primIDs != vuint<M>(-1); }
@@ -79,122 +70,19 @@ namespace embree
     __forceinline const vuint<M>& primID() const { return primIDs; }
     __forceinline unsigned int primID(const size_t i) const { assert(i<M); return primIDs[i]; }
 
-    __forceinline Vec3f& getVertex(const vuint<M>& v, const size_t index, const Scene *const scene) const
-    {
-      const float* vertices = scene->vertices[geomID(index)];
-      return (Vec3f&) vertices[v[index]];
-    }
-
-    template<typename T>
-    __forceinline Vec3<T> getVertex(const vuint<M> &v, const size_t index, const Scene *const scene, const size_t itime, const T& ftime) const
-    {
-      const QuadMesh* mesh = scene->get<QuadMesh>(geomID(index));
-      const float* vertices0 = (const float*) mesh->vertexPtr(0,itime+0);
-      const float* vertices1 = (const float*) mesh->vertexPtr(0,itime+1);
-      const Vec3fa v0 = Vec3fa::loadu(vertices0+v[index]);
-      const Vec3fa v1 = Vec3fa::loadu(vertices1+v[index]);
-      const Vec3<T> p0(v0.x,v0.y,v0.z);
-      const Vec3<T> p1(v1.x,v1.y,v1.z);
-      return lerp(p0,p1,ftime);
-    }
-
-    template<int K, typename T>
-    __forceinline Vec3<T> getVertex(const vbool<K>& valid, const vuint<M>& v, const size_t index, const Scene *const scene, const vint<K>& itime, const T& ftime) const
-    {
-      Vec3<T> p0, p1;
-      const QuadMesh* mesh = scene->get<QuadMesh>(geomID(index));
-
-      for (size_t mask=movemask(valid), i=bsf(mask); mask; mask=btc(mask,i), i=bsf(mask))
-      {
-        const float* vertices0 = (const float*) mesh->vertexPtr(0,itime[i]+0);
-        const float* vertices1 = (const float*) mesh->vertexPtr(0,itime[i]+1);
-        const Vec3fa v0 = Vec3fa::loadu(vertices0+v[index]);
-        const Vec3fa v1 = Vec3fa::loadu(vertices1+v[index]);
-        p0.x[i] = v0.x; p0.y[i] = v0.y; p0.z[i] = v0.z;
-        p1.x[i] = v1.x; p1.y[i] = v1.y; p1.z[i] = v1.z;
-      }
-      return (T(one)-ftime)*p0 + ftime*p1;
-    }
-
-    /* Gather the quads */
-    __forceinline void gather(Vec3vf<M>& p0,
-                              Vec3vf<M>& p1,
-                              Vec3vf<M>& p2,
-                              Vec3vf<M>& p3,
-                              const Scene *const scene) const;
-
-#if defined(__AVX512F__)
-    __forceinline void gather(Vec3vf16& p0,
-                              Vec3vf16& p1,
-                              Vec3vf16& p2,
-                              Vec3vf16& p3,
-                              const Scene *const scene) const;
-#endif
-
-    template<int K>
-    __forceinline void gather(const vbool<K>& valid,
-                              Vec3vf<K>& p0,
-                              Vec3vf<K>& p1,
-                              Vec3vf<K>& p2,
-                              Vec3vf<K>& p3,
-                              const size_t index,
-                              const Scene* const scene,
-                              const vfloat<K>& time) const
-    {
-      const QuadMesh* mesh = scene->get<QuadMesh>(geomID(index));
-
-      vfloat<K> ftime;
-      const vint<K> itime = mesh->timeSegment(time, ftime);
-
-      const size_t first = bsf(movemask(valid));
-      if (likely(all(valid,itime[first] == itime)))
-      {
-        p0 = getVertex(v0, index, scene, itime[first], ftime);
-        p1 = getVertex(v1, index, scene, itime[first], ftime);
-        p2 = getVertex(v2, index, scene, itime[first], ftime);
-        p3 = getVertex(v3, index, scene, itime[first], ftime);
-      }
-      else
-      {
-        p0 = getVertex(valid, v0, index, scene, itime, ftime);
-        p1 = getVertex(valid, v1, index, scene, itime, ftime);
-        p2 = getVertex(valid, v2, index, scene, itime, ftime);
-        p3 = getVertex(valid, v3, index, scene, itime, ftime);
-      }
-    }
-
-    __forceinline void gather(Vec3vf<M>& p0,
-                              Vec3vf<M>& p1,
-                              Vec3vf<M>& p2,
-                              Vec3vf<M>& p3,
-                              const QuadMesh* mesh,
-                              const int itime) const;
-
-    __forceinline void gather(Vec3vf<M>& p0,
-                              Vec3vf<M>& p1,
-                              Vec3vf<M>& p2,
-                              Vec3vf<M>& p3,
-                              const Scene *const scene,
-                              const float time) const;
-
     /* Calculate the bounds of the quads */
     __forceinline const BBox3fa bounds(const Scene *const scene, const size_t itime=0) const
     {
       BBox3fa bounds = empty;
-      for (size_t i=0; i<M && valid(i); i++)
-      {
-        const float* vertices = (const float*) scene->get<QuadMesh>(geomID(i))->vertexPtr(0,itime);
-        bounds.extend(Vec3fa::loadu(vertices+v0[i]));
-        bounds.extend(Vec3fa::loadu(vertices+v1[i]));
-        bounds.extend(Vec3fa::loadu(vertices+v2[i]));
-        bounds.extend(Vec3fa::loadu(vertices+v3[i]));
+      for (size_t i=0; i<M && valid(i); i++) {
+        const QuadMesh* mesh = scene->get<QuadMesh>(geomID(i));
+        bounds.extend(mesh->bounds(primID(i),itime));
       }
       return bounds;
     }
 
     /* Calculate the linear bounds of the primitive */
-    __forceinline LBBox3fa linearBounds(const Scene* const scene, const size_t itime)
-    {
+    __forceinline LBBox3fa linearBounds(const Scene* const scene, const size_t itime) {
       return LBBox3fa(bounds(scene,itime+0),bounds(scene,itime+1));
     }
 
@@ -225,21 +113,23 @@ namespace embree
     __forceinline void fill(const PrimRefT* prims, size_t& begin, size_t end, Scene* scene)
     {
       vuint<M> geomID = -1, primID = -1;
-      vuint<M> v0 = zero, v1 = zero, v2 = zero, v3 = zero;
       const PrimRefT* prim = &prims[begin];
+      vuint<M> v0 = zero, v1 = zero, v2 = zero, v3 = zero;
 
       for (size_t i=0; i<M; i++)
       {
-        const QuadMesh* mesh = scene->get<QuadMesh>(prim->geomID());
-        const QuadMesh::Quad& q = mesh->quad(prim->primID());
         if (begin<end) {
           geomID[i] = prim->geomID();
           primID[i] = prim->primID();
+#if !defined(EMBREE_COMPACT_POLYS)
+          const QuadMesh* mesh = scene->get<QuadMesh>(prim->geomID());
+          const QuadMesh::Quad& q = mesh->quad(prim->primID());
           unsigned int_stride = mesh->vertices0.getStride()/4;
           v0[i] = q.v[0] * int_stride;
           v1[i] = q.v[1] * int_stride;
           v2[i] = q.v[2] * int_stride;
           v3[i] = q.v[3] * int_stride;
+#endif
           begin++;
         } else {
           assert(i);
@@ -254,7 +144,6 @@ namespace embree
         }
         if (begin<end) prim = &prims[begin];
       }
-
       new (this) QuadMi(v0,v1,v2,v3,geomID,primID); // FIXME: use non temporal store
     }
 
@@ -270,10 +159,230 @@ namespace embree
       return linearBounds(scene, time_range);
     }
 
-    friend std::ostream& operator<<(std::ostream& cout, const QuadMi& quad) {
-      return cout << "QuadMi<" << M << ">( v0 = " << quad.v0 << ", v1 = " << quad.v1 << ", v2 = " << quad.v2 << ", v3 = " << quad.v3 << ", geomID = " << quad.geomIDs << ", primID = " << quad.primIDs << " )";
+    friend embree_ostream operator<<(embree_ostream cout, const QuadMi& quad) {
+      return cout << "QuadMi<" << M << ">( "
+#if !defined(EMBREE_COMPACT_POLYS)
+                  << "v0 = " << quad.v0_ << ", v1 = " << quad.v1_ << ", v2 = " << quad.v2_ << ", v3 = " << quad.v3_ << ", "
+#endif
+                  << "geomID = " << quad.geomIDs << ", primID = " << quad.primIDs << " )";
     }
 
+  protected:
+#if !defined(EMBREE_COMPACT_POLYS)
+    vuint<M> v0_;         // 4 byte offset of 1st vertex
+    vuint<M> v1_;         // 4 byte offset of 2nd vertex
+    vuint<M> v2_;         // 4 byte offset of 3rd vertex
+    vuint<M> v3_;         // 4 byte offset of 4th vertex
+#endif
+    vuint<M> geomIDs;    // geometry ID of mesh
+    vuint<M> primIDs;    // primitive ID of primitive inside mesh
+  };
+
+  namespace isa
+  {
+    
+  template<int M>
+    struct QuadMi : public embree::QuadMi<M>
+  {
+#if !defined(EMBREE_COMPACT_POLYS)
+    using embree::QuadMi<M>::v0_;
+    using embree::QuadMi<M>::v1_;
+    using embree::QuadMi<M>::v2_;
+    using embree::QuadMi<M>::v3_;
+#endif
+    using embree::QuadMi<M>::geomIDs;
+    using embree::QuadMi<M>::primIDs;
+    using embree::QuadMi<M>::geomID;
+    using embree::QuadMi<M>::primID;
+    using embree::QuadMi<M>::valid;
+    
+    template<int vid>
+    __forceinline Vec3f getVertex(const size_t index, const Scene *const scene) const
+    {
+#if defined(EMBREE_COMPACT_POLYS)
+      const QuadMesh* mesh = scene->get<QuadMesh>(geomID(index));
+      const QuadMesh::Quad& quad = mesh->quad(primID(index));
+      return (Vec3f) mesh->vertices[0][quad.v[vid]];
+#else
+      const vuint<M>& v = getVertexOffset<vid>();
+      const float* vertices = scene->vertices[geomID(index)];
+      return (Vec3f&) vertices[v[index]];
+#endif
+    }
+
+    template<int vid, typename T>
+    __forceinline Vec3<T> getVertex(const size_t index, const Scene *const scene, const size_t itime, const T& ftime) const
+    {
+#if defined(EMBREE_COMPACT_POLYS)
+      const QuadMesh* mesh = scene->get<QuadMesh>(geomID(index));
+      const QuadMesh::Quad& quad = mesh->quad(primID(index));
+      const Vec3fa v0 = mesh->vertices[itime+0][quad.v[vid]];
+      const Vec3fa v1 = mesh->vertices[itime+1][quad.v[vid]];
+#else
+      const vuint<M>& v = getVertexOffset<vid>();
+      const QuadMesh* mesh = scene->get<QuadMesh>(geomID(index));
+      const float* vertices0 = (const float*) mesh->vertexPtr(0,itime+0);
+      const float* vertices1 = (const float*) mesh->vertexPtr(0,itime+1);
+      const Vec3fa v0 = Vec3fa::loadu(vertices0+v[index]);
+      const Vec3fa v1 = Vec3fa::loadu(vertices1+v[index]);
+#endif
+      const Vec3<T> p0(v0.x,v0.y,v0.z);
+      const Vec3<T> p1(v1.x,v1.y,v1.z);
+      return lerp(p0,p1,ftime);
+    }
+
+    template<int vid, int K, typename T>
+    __forceinline Vec3<T> getVertex(const vbool<K>& valid, const size_t index, const Scene *const scene, const vint<K>& itime, const T& ftime) const
+    {
+      Vec3<T> p0, p1;
+      const QuadMesh* mesh = scene->get<QuadMesh>(geomID(index));
+
+      for (size_t mask=movemask(valid), i=bsf(mask); mask; mask=btc(mask,i), i=bsf(mask))
+      {
+#if defined(EMBREE_COMPACT_POLYS)
+        const QuadMesh::Quad& quad = mesh->quad(primID(index));
+        const Vec3fa v0 = mesh->vertices[itime[i]+0][quad.v[vid]];
+        const Vec3fa v1 = mesh->vertices[itime[i]+1][quad.v[vid]];
+#else
+        const vuint<M>& v = getVertexOffset<vid>();
+        const float* vertices0 = (const float*) mesh->vertexPtr(0,itime[i]+0);
+        const float* vertices1 = (const float*) mesh->vertexPtr(0,itime[i]+1);
+        const Vec3fa v0 = Vec3fa::loadu(vertices0+v[index]);
+        const Vec3fa v1 = Vec3fa::loadu(vertices1+v[index]);
+#endif
+        p0.x[i] = v0.x; p0.y[i] = v0.y; p0.z[i] = v0.z;
+        p1.x[i] = v1.x; p1.y[i] = v1.y; p1.z[i] = v1.z;
+      }
+      return (T(one)-ftime)*p0 + ftime*p1;
+    }
+
+    struct Quad {
+      vfloat4 v0,v1,v2,v3;
+    };
+
+#if defined(EMBREE_COMPACT_POLYS)
+    
+    __forceinline Quad loadQuad(const int i, const Scene* const scene) const 
+    {
+      const unsigned int geomID = geomIDs[i];
+      const unsigned int primID = primIDs[i];
+      if (unlikely(primID == -1)) return { zero, zero, zero, zero };
+      const QuadMesh* mesh = scene->get<QuadMesh>(geomID);
+      const QuadMesh::Quad& quad = mesh->quad(primID);
+      const vfloat4 v0 = (vfloat4) mesh->vertices0[quad.v[0]];
+      const vfloat4 v1 = (vfloat4) mesh->vertices0[quad.v[1]];
+      const vfloat4 v2 = (vfloat4) mesh->vertices0[quad.v[2]];
+      const vfloat4 v3 = (vfloat4) mesh->vertices0[quad.v[3]];
+      return { v0, v1, v2, v3 };
+    }
+
+    __forceinline Quad loadQuad(const int i, const int itime, const Scene* const scene) const 
+    {
+      const unsigned int geomID = geomIDs[i];
+      const unsigned int primID = primIDs[i];
+      if (unlikely(primID == -1)) return { zero, zero, zero, zero };
+      const QuadMesh* mesh = scene->get<QuadMesh>(geomID);
+      const QuadMesh::Quad& quad = mesh->quad(primID);
+      const vfloat4 v0 = (vfloat4) mesh->vertices[itime][quad.v[0]];
+      const vfloat4 v1 = (vfloat4) mesh->vertices[itime][quad.v[1]];
+      const vfloat4 v2 = (vfloat4) mesh->vertices[itime][quad.v[2]];
+      const vfloat4 v3 = (vfloat4) mesh->vertices[itime][quad.v[3]];
+      return { v0, v1, v2, v3 };
+    }
+    
+#else
+
+    __forceinline Quad loadQuad(const int i, const Scene* const scene) const 
+    {
+      const float* vertices = scene->vertices[geomID(i)];
+      const vfloat4 v0 = vfloat4::loadu(vertices + v0_[i]);
+      const vfloat4 v1 = vfloat4::loadu(vertices + v1_[i]);
+      const vfloat4 v2 = vfloat4::loadu(vertices + v2_[i]);
+      const vfloat4 v3 = vfloat4::loadu(vertices + v3_[i]);
+      return { v0, v1, v2, v3 };
+    }
+
+    __forceinline Quad loadQuad(const int i, const int itime, const Scene* const scene) const 
+    {
+      const unsigned int geomID = geomIDs[i];
+      const QuadMesh* mesh = scene->get<QuadMesh>(geomID);
+      const float* vertices = (const float*) mesh->vertexPtr(0,itime);
+      const vfloat4 v0 = vfloat4::loadu(vertices + v0_[i]);
+      const vfloat4 v1 = vfloat4::loadu(vertices + v1_[i]);
+      const vfloat4 v2 = vfloat4::loadu(vertices + v2_[i]);
+      const vfloat4 v3 = vfloat4::loadu(vertices + v3_[i]);
+      return { v0, v1, v2, v3 };
+    }
+    
+#endif
+
+    /* Gather the quads */
+    __forceinline void gather(Vec3vf<M>& p0,
+                              Vec3vf<M>& p1,
+                              Vec3vf<M>& p2,
+                              Vec3vf<M>& p3,
+                              const Scene *const scene) const;
+
+#if defined(__AVX512F__)
+    __forceinline void gather(Vec3vf16& p0,
+                              Vec3vf16& p1,
+                              Vec3vf16& p2,
+                              Vec3vf16& p3,
+                              const Scene *const scene) const;
+#endif
+
+    template<int K>
+#if defined(__INTEL_COMPILER) && (__INTEL_COMPILER < 2000) // workaround for compiler bug in ICC 2019
+    __noinline
+#else
+    __forceinline
+#endif
+    void gather(const vbool<K>& valid,
+      Vec3vf<K>& p0,
+      Vec3vf<K>& p1,
+      Vec3vf<K>& p2,
+      Vec3vf<K>& p3,
+      const size_t index,
+      const Scene* const scene,
+      const vfloat<K>& time) const
+    {
+      const QuadMesh* mesh = scene->get<QuadMesh>(geomID(index));
+
+      vfloat<K> ftime;
+      const vint<K> itime = mesh->timeSegment(time, ftime);
+
+      const size_t first = bsf(movemask(valid));
+      if (likely(all(valid,itime[first] == itime)))
+      {
+        p0 = getVertex<0>(index, scene, itime[first], ftime);
+        p1 = getVertex<1>(index, scene, itime[first], ftime);
+        p2 = getVertex<2>(index, scene, itime[first], ftime);
+        p3 = getVertex<3>(index, scene, itime[first], ftime);
+      }
+      else
+      {
+        p0 = getVertex<0>(valid, index, scene, itime, ftime);
+        p1 = getVertex<1>(valid, index, scene, itime, ftime);
+        p2 = getVertex<2>(valid, index, scene, itime, ftime);
+        p3 = getVertex<3>(valid, index, scene, itime, ftime);
+      }
+    }
+
+    __forceinline void gather(Vec3vf<M>& p0,
+                              Vec3vf<M>& p1,
+                              Vec3vf<M>& p2,
+                              Vec3vf<M>& p3,
+                              const QuadMesh* mesh,
+                              const Scene *const scene,
+                              const int itime) const;
+
+    __forceinline void gather(Vec3vf<M>& p0,
+                              Vec3vf<M>& p1,
+                              Vec3vf<M>& p2,
+                              Vec3vf<M>& p3,
+                              const Scene *const scene,
+                              const float time) const;
+
     /* Updates the primitive */
     __forceinline BBox3fa update(QuadMesh* mesh)
     {
@@ -292,17 +401,19 @@ namespace embree
       return bounds;
     }
 
-
-  public:
-    vuint<M> v0;         // 4 byte offset of 1st vertex
-    vuint<M> v1;         // 4 byte offset of 2nd vertex
-    vuint<M> v2;         // 4 byte offset of 3rd vertex
-    vuint<M> v3;         // 4 byte offset of 4th vertex
   private:
-    vuint<M> geomIDs;    // geometry ID of mesh
-    vuint<M> primIDs;    // primitive ID of primitive inside mesh
+#if !defined(EMBREE_COMPACT_POLYS)
+    template<int N> const vuint<M>& getVertexOffset() const;
+#endif
   };
 
+#if !defined(EMBREE_COMPACT_POLYS)
+  template<> template<> __forceinline const vuint<4>& QuadMi<4>::getVertexOffset<0>() const { return v0_; }
+  template<> template<> __forceinline const vuint<4>& QuadMi<4>::getVertexOffset<1>() const { return v1_; }
+  template<> template<> __forceinline const vuint<4>& QuadMi<4>::getVertexOffset<2>() const { return v2_; }
+  template<> template<> __forceinline const vuint<4>& QuadMi<4>::getVertexOffset<3>() const { return v3_; }
+#endif
+
   template<>
   __forceinline void QuadMi<4>::gather(Vec3vf4& p0,
                                        Vec3vf4& p1,
@@ -312,120 +423,35 @@ namespace embree
   {
     prefetchL1(((char*)this)+0*64);
     prefetchL1(((char*)this)+1*64);
-    const float* vertices0 = scene->vertices[geomID(0)];
-    const float* vertices1 = scene->vertices[geomID(1)];
-    const float* vertices2 = scene->vertices[geomID(2)];
-    const float* vertices3 = scene->vertices[geomID(3)];
-    const vfloat4 a0 = vfloat4::loadu(vertices0 + v0[0]);
-    const vfloat4 a1 = vfloat4::loadu(vertices1 + v0[1]);
-    const vfloat4 a2 = vfloat4::loadu(vertices2 + v0[2]);
-    const vfloat4 a3 = vfloat4::loadu(vertices3 + v0[3]);
-    const vfloat4 b0 = vfloat4::loadu(vertices0 + v1[0]);
-    const vfloat4 b1 = vfloat4::loadu(vertices1 + v1[1]);
-    const vfloat4 b2 = vfloat4::loadu(vertices2 + v1[2]);
-    const vfloat4 b3 = vfloat4::loadu(vertices3 + v1[3]);
-    const vfloat4 c0 = vfloat4::loadu(vertices0 + v2[0]);
-    const vfloat4 c1 = vfloat4::loadu(vertices1 + v2[1]);
-    const vfloat4 c2 = vfloat4::loadu(vertices2 + v2[2]);
-    const vfloat4 c3 = vfloat4::loadu(vertices3 + v2[3]);
-    const vfloat4 d0 = vfloat4::loadu(vertices0 + v3[0]);
-    const vfloat4 d1 = vfloat4::loadu(vertices1 + v3[1]);
-    const vfloat4 d2 = vfloat4::loadu(vertices2 + v3[2]);
-    const vfloat4 d3 = vfloat4::loadu(vertices3 + v3[3]);
-    transpose(a0,a1,a2,a3,p0.x,p0.y,p0.z);
-    transpose(b0,b1,b2,b3,p1.x,p1.y,p1.z);
-    transpose(c0,c1,c2,c3,p2.x,p2.y,p2.z);
-    transpose(d0,d1,d2,d3,p3.x,p3.y,p3.z);
+    const Quad tri0 = loadQuad(0,scene);
+    const Quad tri1 = loadQuad(1,scene);
+    const Quad tri2 = loadQuad(2,scene);
+    const Quad tri3 = loadQuad(3,scene);
+    transpose(tri0.v0,tri1.v0,tri2.v0,tri3.v0,p0.x,p0.y,p0.z);
+    transpose(tri0.v1,tri1.v1,tri2.v1,tri3.v1,p1.x,p1.y,p1.z);
+    transpose(tri0.v2,tri1.v2,tri2.v2,tri3.v2,p2.x,p2.y,p2.z);
+    transpose(tri0.v3,tri1.v3,tri2.v3,tri3.v3,p3.x,p3.y,p3.z);
   }
 
-#if defined(__AVX512F__)
-  template<>
-  __forceinline void QuadMi<4>::gather(Vec3vf16& p0,
-                                       Vec3vf16& p1,
-                                       Vec3vf16& p2,
-                                       Vec3vf16& p3,
-                                       const Scene *const scene) const // FIXME: why do we have this special path here and not for triangles?
-  {
-    const vint16 perm(0,4,8,12,1,5,9,13,2,6,10,14,3,7,11,15);
-    const float* vertices0 = scene->vertices[geomID(0)];
-    const float* vertices1 = scene->vertices[geomID(1)];
-    const float* vertices2 = scene->vertices[geomID(2)];
-    const float* vertices3 = scene->vertices[geomID(3)];
-
-    const vfloat4 a0 = vfloat4::loadu(vertices0 + v0[0]);
-    const vfloat4 a1 = vfloat4::loadu(vertices1 + v0[1]);
-    const vfloat4 a2 = vfloat4::loadu(vertices2 + v0[2]);
-    const vfloat4 a3 = vfloat4::loadu(vertices3 + v0[3]);
-    const vfloat16 _p0(permute(vfloat16(a0,a1,a2,a3),perm));
-
-    const vfloat4 b0 = vfloat4::loadu(vertices0 + v1[0]);
-    const vfloat4 b1 = vfloat4::loadu(vertices1 + v1[1]);
-    const vfloat4 b2 = vfloat4::loadu(vertices2 + v1[2]);
-    const vfloat4 b3 = vfloat4::loadu(vertices3 + v1[3]);
-    const vfloat16 _p1(permute(vfloat16(b0,b1,b2,b3),perm));
-
-    const vfloat4 c0 = vfloat4::loadu(vertices0 + v2[0]);
-    const vfloat4 c1 = vfloat4::loadu(vertices1 + v2[1]);
-    const vfloat4 c2 = vfloat4::loadu(vertices2 + v2[2]);
-    const vfloat4 c3 = vfloat4::loadu(vertices3 + v2[3]);
-    const vfloat16 _p2(permute(vfloat16(c0,c1,c2,c3),perm));
-
-    const vfloat4 d0 = vfloat4::loadu(vertices0 + v3[0]);
-    const vfloat4 d1 = vfloat4::loadu(vertices1 + v3[1]);
-    const vfloat4 d2 = vfloat4::loadu(vertices2 + v3[2]);
-    const vfloat4 d3 = vfloat4::loadu(vertices3 + v3[3]);
-    const vfloat16 _p3(permute(vfloat16(d0,d1,d2,d3),perm));
-
-    p0.x = shuffle4<0>(_p0);
-    p0.y = shuffle4<1>(_p0);
-    p0.z = shuffle4<2>(_p0);
-
-    p1.x = shuffle4<0>(_p1);
-    p1.y = shuffle4<1>(_p1);
-    p1.z = shuffle4<2>(_p1);
-
-    p2.x = shuffle4<0>(_p2);
-    p2.y = shuffle4<1>(_p2);
-    p2.z = shuffle4<2>(_p2);
-
-    p3.x = shuffle4<0>(_p3);
-    p3.y = shuffle4<1>(_p3);
-    p3.z = shuffle4<2>(_p3);
-  }
-#endif
-
   template<>
   __forceinline void QuadMi<4>::gather(Vec3vf4& p0,
                                        Vec3vf4& p1,
                                        Vec3vf4& p2,
                                        Vec3vf4& p3,
                                        const QuadMesh* mesh,
+                                       const Scene *const scene,
                                        const int itime) const
   {
-    const float* vertices0 = (const float*) mesh->vertexPtr(0,itime);
-    const float* vertices1 = (const float*) mesh->vertexPtr(0,itime);
-    const float* vertices2 = (const float*) mesh->vertexPtr(0,itime);
-    const float* vertices3 = (const float*) mesh->vertexPtr(0,itime);
-    const vfloat4 a0 = vfloat4::loadu(vertices0 + v0[0]);
-    const vfloat4 a1 = vfloat4::loadu(vertices1 + v0[1]);
-    const vfloat4 a2 = vfloat4::loadu(vertices2 + v0[2]);
-    const vfloat4 a3 = vfloat4::loadu(vertices3 + v0[3]);
-    const vfloat4 b0 = vfloat4::loadu(vertices0 + v1[0]);
-    const vfloat4 b1 = vfloat4::loadu(vertices1 + v1[1]);
-    const vfloat4 b2 = vfloat4::loadu(vertices2 + v1[2]);
-    const vfloat4 b3 = vfloat4::loadu(vertices3 + v1[3]);
-    const vfloat4 c0 = vfloat4::loadu(vertices0 + v2[0]);
-    const vfloat4 c1 = vfloat4::loadu(vertices1 + v2[1]);
-    const vfloat4 c2 = vfloat4::loadu(vertices2 + v2[2]);
-    const vfloat4 c3 = vfloat4::loadu(vertices3 + v2[3]);
-    const vfloat4 d0 = vfloat4::loadu(vertices0 + v3[0]);
-    const vfloat4 d1 = vfloat4::loadu(vertices1 + v3[1]);
-    const vfloat4 d2 = vfloat4::loadu(vertices2 + v3[2]);
-    const vfloat4 d3 = vfloat4::loadu(vertices3 + v3[3]);
-    transpose(a0,a1,a2,a3,p0.x,p0.y,p0.z);
-    transpose(b0,b1,b2,b3,p1.x,p1.y,p1.z);
-    transpose(c0,c1,c2,c3,p2.x,p2.y,p2.z);
-    transpose(d0,d1,d2,d3,p3.x,p3.y,p3.z);
+    // FIXME: for trianglei there all geometries are identical, is this the case here too?
+    
+    const Quad tri0 = loadQuad(0,itime,scene);
+    const Quad tri1 = loadQuad(1,itime,scene);
+    const Quad tri2 = loadQuad(2,itime,scene);
+    const Quad tri3 = loadQuad(3,itime,scene);
+    transpose(tri0.v0,tri1.v0,tri2.v0,tri3.v0,p0.x,p0.y,p0.z);
+    transpose(tri0.v1,tri1.v1,tri2.v1,tri3.v1,p1.x,p1.y,p1.z);
+    transpose(tri0.v2,tri1.v2,tri2.v2,tri3.v2,p2.x,p2.y,p2.z);
+    transpose(tri0.v3,tri1.v3,tri2.v3,tri3.v3,p3.x,p3.y,p3.z);
   }
 
   template<>
@@ -441,13 +467,14 @@ namespace embree
     float ftime;
     const int itime = mesh->timeSegment(time, ftime);
 
-    Vec3vf4 a0,a1,a2,a3; gather(a0,a1,a2,a3,mesh,itime);
-    Vec3vf4 b0,b1,b2,b3; gather(b0,b1,b2,b3,mesh,itime+1);
+    Vec3vf4 a0,a1,a2,a3; gather(a0,a1,a2,a3,mesh,scene,itime);
+    Vec3vf4 b0,b1,b2,b3; gather(b0,b1,b2,b3,mesh,scene,itime+1);
     p0 = lerp(a0,b0,vfloat4(ftime));
     p1 = lerp(a1,b1,vfloat4(ftime));
     p2 = lerp(a2,b2,vfloat4(ftime));
     p3 = lerp(a3,b3,vfloat4(ftime));
   }
+  }
 
   template<int M>
   typename QuadMi<M>::Type QuadMi<M>::type;
