@@ -1,18 +1,5 @@
-// ======================================================================== //
-// Copyright 2009-2020 Intel Corporation                                    //
-//                                                                          //
-// Licensed under the Apache License, Version 2.0 (the "License");          //
-// you may not use this file except in compliance with the License.         //
-// You may obtain a copy of the License at                                  //
-//                                                                          //
-//     http://www.apache.org/licenses/LICENSE-2.0                           //
-//                                                                          //
-// Unless required by applicable law or agreed to in writing, software      //
-// distributed under the License is distributed on an "AS IS" BASIS,        //
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. //
-// See the License for the specific language governing permissions and      //
-// limitations under the License.                                           //
-// ======================================================================== //
+// Copyright 2009-2020 Intel Corporation
+// SPDX-License-Identifier: Apache-2.0
 
 #include "../common/tutorial/tutorial.h"
 #include "../common/tutorial/tutorial_device.h"
@@ -145,11 +132,11 @@ namespace embree
   /* prints the bvh4.triangle4v data structure */
   void print_bvh4_triangle4v(BVH4::NodeRef node, size_t depth)
   {
-    if (node.isAlignedNode())
+    if (node.isAABBNode())
     {
-      BVH4::AlignedNode* n = node.alignedNode();
+      BVH4::AABBNode* n = node.getAABBNode();
       
-      std::cout << "AlignedNode {" << std::endl;
+      std::cout << "AABBNode {" << std::endl;
       for (size_t i=0; i<4; i++)
       {
         for (size_t k=0; k<depth; k++) std::cout << "  ";
