@@ -1,18 +1,5 @@
-// ======================================================================== //
-// Copyright 2009-2020 Intel Corporation                                    //
-//                                                                          //
-// Licensed under the Apache License, Version 2.0 (the "License");          //
-// you may not use this file except in compliance with the License.         //
-// You may obtain a copy of the License at                                  //
-//                                                                          //
-//     http://www.apache.org/licenses/LICENSE-2.0                           //
-//                                                                          //
-// Unless required by applicable law or agreed to in writing, software      //
-// distributed under the License is distributed on an "AS IS" BASIS,        //
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. //
-// See the License for the specific language governing permissions and      //
-// limitations under the License.                                           //
-// ======================================================================== //
+// Copyright 2009-2020 Intel Corporation
+// SPDX-License-Identifier: Apache-2.0
 
 #include "../common/tutorial/tutorial_device.h"
 
@@ -40,8 +27,8 @@ inline void invalidateRay(Ray& ray)
   // in streamed mode, active state of lanes is forgotten between ray
   // generation and traversal.
   {
-    ray.org = Vec3fa(0.f);
-    ray.dir = Vec3fa(0.f);
+    ray.org = Vec3ff(0.f);
+    ray.dir = Vec3ff(0.f);
     ray.tnear() = pos_inf;
     ray.tfar = neg_inf;
   }
@@ -129,12 +116,12 @@ void instanceIntersectFunc(const RTCIntersectFunctionNArguments* args)
   
   Ray *ray = (Ray*)rays;
   const Instance* instance = (const Instance*)ptr;
-  const Vec3fa ray_org = ray->org;
-  const Vec3fa ray_dir = ray->dir;
+  const Vec3ff ray_org = ray->org;
+  const Vec3ff ray_dir = ray->dir;
   const float ray_tnear = ray->tnear();
   const float ray_tfar  = ray->tfar;
-  ray->org = xfmPoint (instance->world2local,ray_org);
-  ray->dir = xfmVector(instance->world2local,ray_dir);
+  ray->org = Vec3ff(xfmPoint (instance->world2local,ray_org));
+  ray->dir = Vec3ff(xfmVector(instance->world2local,ray_dir));
   ray->tnear() = ray_tnear;
   ray->tfar  = ray_tfar;
   pushInstanceId(context, instance->userID);
@@ -158,12 +145,12 @@ void instanceOccludedFunc(const RTCOccludedFunctionNArguments* args)
   
   Ray *ray = (Ray*)rays;
   const Instance* instance = (const Instance*)ptr;
-  const Vec3fa ray_org = ray->org;
-  const Vec3fa ray_dir = ray->dir;
+  const Vec3ff ray_org = ray->org;
+  const Vec3ff ray_dir = ray->dir;
   const float ray_tnear = ray->tnear();
   const float ray_tfar  = ray->tfar;
-  ray->org    = xfmPoint (instance->world2local,ray_org);
-  ray->dir    = xfmVector(instance->world2local,ray_dir);
+  ray->org    = Vec3ff(xfmPoint (instance->world2local,ray_org));
+  ray->dir    = Vec3ff(xfmVector(instance->world2local,ray_dir));
   ray->tnear()  = ray_tnear;
   ray->tfar   = ray_tfar;
   pushInstanceId(context, instance->userID);
@@ -206,8 +193,8 @@ void instanceIntersectFuncN(const RTCIntersectFunctionNArguments* args)
     invalidateRay(ray);
     const Vec3fa ray_org = Vec3fa(RTCRayN_org_x(rays,N,ui),RTCRayN_org_y(rays,N,ui),RTCRayN_org_z(rays,N,ui));
     const Vec3fa ray_dir = Vec3fa(RTCRayN_dir_x(rays,N,ui),RTCRayN_dir_y(rays,N,ui),RTCRayN_dir_z(rays,N,ui));
-    ray.org = xfmPoint (instance->world2local,ray_org);
-    ray.dir = xfmVector(instance->world2local,ray_dir);
+    ray.org = Vec3ff(xfmPoint (instance->world2local,ray_org));
+    ray.dir = Vec3ff(xfmVector(instance->world2local,ray_dir));
     ray.tnear() = RTCRayN_tnear(rays,N,ui);
     ray.tfar  = RTCRayN_tfar(rays,N,ui);
     ray.time()  = RTCRayN_time(rays,N,ui);
@@ -254,8 +241,8 @@ void instanceOccludedFuncN(const RTCOccludedFunctionNArguments* args)
     invalidateRay(ray);
     const Vec3fa ray_org = Vec3fa(RTCRayN_org_x(rays,N,ui),RTCRayN_org_y(rays,N,ui),RTCRayN_org_z(rays,N,ui));
     const Vec3fa ray_dir = Vec3fa(RTCRayN_dir_x(rays,N,ui),RTCRayN_dir_y(rays,N,ui),RTCRayN_dir_z(rays,N,ui));
-    ray.org = xfmPoint (instance->world2local,ray_org);
-    ray.dir = xfmVector(instance->world2local,ray_dir);
+    ray.org = Vec3ff(xfmPoint (instance->world2local,ray_org));
+    ray.dir = Vec3ff(xfmVector(instance->world2local,ray_dir));
     ray.tnear() = RTCRayN_tnear(rays,N,ui);
     ray.tfar = RTCRayN_tfar(rays,N,ui);
     ray.time()  = RTCRayN_time(rays,N,ui);
@@ -966,6 +953,7 @@ extern "C" void device_init (char* cfg)
 
   /* create scene with 4 analytical spheres */
   g_scene0 = rtcNewScene(g_device);
+  rtcSetSceneBuildQuality(g_scene0,RTC_BUILD_QUALITY_LOW);
   g_spheres = createAnalyticalSpheres(g_scene0,4);
   g_spheres[0].p = Vec3fa( 0, 0,+1); g_spheres[0].r = 0.5f;
   g_spheres[1].p = Vec3fa(+1, 0, 0); g_spheres[1].r = 0.5f;
@@ -1022,11 +1010,6 @@ extern "C" void device_init (char* cfg)
   colors[4][1] = Vec3fa(1.0f, 1.0f, 1.0f);
   colors[4][2] = Vec3fa(1.0f, 1.0f, 1.0f);
   colors[4][3] = Vec3fa(1.0f, 1.0f, 1.0f);
-
-  /* set start render mode */
-  if (g_mode == MODE_NORMAL) renderTile = renderTileStandard;
-  else                       renderTile = renderTileStandardStream;
-  key_pressed_handler = device_key_pressed_default;
 }
 
 inline Vec3fa face_forward(const Vec3fa& dir, const Vec3fa& _Ng) {
@@ -1183,8 +1166,8 @@ void renderTileStandardStream(int taskIndex,
     /* invalidate shadow rays by default */
     Ray& shadow = shadow_stream[N];
     {
-      shadow.org = Vec3fa(0.f);
-      shadow.dir = Vec3fa(0.f);
+      shadow.org = Vec3ff(0.f);
+      shadow.dir = Vec3ff(0.f);
       shadow.tnear() = (float)(pos_inf);
       shadow.tfar  = (float)(neg_inf);
     }
@@ -1276,9 +1259,30 @@ void renderTileTask (int taskIndex, int threadIndex, int* pixels,
                          const int numTilesX,
                          const int numTilesY)
 {
-  renderTile(taskIndex,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
+  if (g_mode == MODE_NORMAL)
+    renderTileStandard(taskIndex,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
+  else
+    renderTileStandardStream(taskIndex,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
+  
 }
 
+extern "C" void renderFrameStandard (int* pixels,
+                          const unsigned int width,
+                          const unsigned int height,
+                          const float time,
+                          const ISPCCamera& camera)
+{
+  /* render all pixels */
+  const int numTilesX = (width +TILE_SIZE_X-1)/TILE_SIZE_X;
+  const int numTilesY = (height+TILE_SIZE_Y-1)/TILE_SIZE_Y;
+  parallel_for(size_t(0),size_t(numTilesX*numTilesY),[&](const range<size_t>& range) {
+    const int threadIndex = (int)TaskScheduler::threadIndex();
+    for (size_t i=range.begin(); i<range.end(); i++)
+      renderTileTask((int)i,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
+  }); 
+}
+
+
 /* called by the C++ code to render */
 extern "C" void device_render (int* pixels,
                            const unsigned int width,
@@ -1307,15 +1311,6 @@ extern "C" void device_render (int* pixels,
   updateInstance(g_scene,g_instance[2]);
   updateInstance(g_scene,g_instance[3]);
   rtcCommitScene (g_scene);
-
-  /* render all pixels */
-  const int numTilesX = (width +TILE_SIZE_X-1)/TILE_SIZE_X;
-  const int numTilesY = (height+TILE_SIZE_Y-1)/TILE_SIZE_Y;
-  parallel_for(size_t(0),size_t(numTilesX*numTilesY),[&](const range<size_t>& range) {
-    const int threadIndex = (int)TaskScheduler::threadIndex();
-    for (size_t i=range.begin(); i<range.end(); i++)
-      renderTileTask((int)i,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
-  }); 
 }
 
 /* called by the C++ code for cleanup */
